#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# pyDialer - "A simple and handy frontend for WvDial, fork of pyWvDial"
# Copyright (C) 2008 Giacomo Lozito
# Copyright (C) 2009 Aliaksandr Stelmachonak
# Licensed under GNU General Public License v2

import os.path
import ConfigParser

class WvDialConfig:

  class ParsingError(Exception):
    pass

  class ConflictingOptionsError(Exception):
    pass

  class SaveToFileError(Exception):
    pass

  class LoadFromFileError(Exception):
    pass

  def __init__(self, cfgFilePath=None):
    self.config = ConfigParser.SafeConfigParser()
    self.default_file = None
    self.section_prefix = 'Dialer '
    self.section_prefix_len = len(self.section_prefix)
    if ( cfgFilePath ):
      self.set_default_file(cfgFilePath)

  def empty_default(self):
    # empty current config
    sections = self.config.sections()
    for section in sections:
      self.config.remove_section(section)
    # and add default section
    self.add_section('Defaults')

  def set_default_file(self, cfgFilePath):
    self.default_file = os.path.expanduser(cfgFilePath)

  def get_default_file(self):
    return self.default_file

  def load_from_file(self, cfgFilePath=None):
    if ( not cfgFilePath ) and ( self.default_file ):
      cfgFilePath = self.default_file
    if ( cfgFilePath ):
      try:
        self.config.read(cfgFilePath)
      except (ConfigParser.MissingSectionHeaderError, ConfigParser.ParsingError):
        raise WvDialConfig.ParsingError()
      self.fix_config()
    else:
      raise WvDialConfig.LoadFromFileError()

  def save_to_file(self, cfgFilePath=None):
    if ( not cfgFilePath ):
      if ( self.default_file ):
        cfgFilePath = self.default_file
      else:
        raise WvDialConfig.SaveToFileError()
    else:
      cfgFilePath = os.path.expanduser(cfgFilePath)
    try:
      file_obj = open(cfgFilePath, 'w')
    except IOError:
      raise WvDialConfig.SaveToFileError()
    else:
      self.config.write(file_obj)
      file_obj.close()

  def fix_config(self):
    # fix a config file annoyance: some config files have the first modem init string in
    # the 'Init' option, while other config files use 'Init1'; wvdial can read either one or
    # the other from the config file, but not together (they have the same meaning so only
    # 'Init' would be executed if both are present in the config file);
    # according to man page, 'Init1' should be the correct choice; so, pick any 'Init' option
    # and move its content to 'Init1' if the latter doesn't exist; if both 'Init' and 'Init1'
    # exists (which means someone wrote a wrong config file that won't behave as intended),
    # raise a ConflictingOptionsError exception
    sections = self.get_sections()
    for section in sections:
      section_f = self.section_prefix + section
      if ( self.config.has_option(section_f, 'Init') ):
        if ( not self.config.has_option(section_f, 'Init1') ):
          # ok, move Init into Init1
          value = self.get_value(section, 'Init')
          self.set_value(section, 'Init1', value)
          self.unset_value(section, 'Init')
        else:
          # ugh... both Init and Init1 are around... let's fail then
          raise WvDialConfig.ConflictingOptionsError(section, 'Init', 'Init1')

  def set_value(self, section, option, value):
    section_f = self.section_prefix + section
    if ( self.config.has_section(section_f) ):
      self.config.set(section_f, option, value)
    else:
      self.add_section(section)
      self.config.set(section_f, option, value)

  def unset_value(self, section, option):
    section = self.section_prefix + section
    if ( self.config.has_section(section) ):
      self.config.remove_option(section, option)

  def get_value(self, section, option):
    try:
      value = self.config.get(self.section_prefix + section, option)
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
      return None
    else:
      return value

  def add_section(self, section):
    section = self.section_prefix + section
    if ( not self.config.has_section(section) ):
      self.config.add_section(section)

  def remove_section(self, section):
    section = self.section_prefix + section
    self.config.remove_section(section)

  def get_sections(self):
    sections = self.config.sections()
    valid_sections = []
    for section in sections:
      # check valid sections in format [SECTION_PREFIX NAME] (for wvdial: [Dialer Name])
      if (( section.startswith(self.section_prefix) ) and ( not section[(self.section_prefix_len - 1):].isspace() )):
        valid_sections.append(section[(self.section_prefix_len):].strip())
    return valid_sections
